<?php

namespace Drp\LdapAuthentication;

use Adldap\Laravel\AdldapAuthUserProvider;
use Adldap\Models\User;
use Illuminate\Contracts\Hashing\Hasher;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Support\Str;
use Illuminate\Support\Arr;

/**
 * Class DrpldapAuthUserProvider
 * @package  Drp\LdapAuthentication
 */
class DrpldapAuthUserProvider extends AdldapAuthUserProvider
{
    /**
     * @var
     */
    private $allowedGroups;

    /**
     * Create a new database user provider.
     * @param Hasher $hasher
     * @param  string $model
     * @param $allowedGroups
     */
    public function __construct(Hasher $hasher, $model, $allowedGroups)
    {
        $this->model = $model;
        $this->hasher = $hasher;
        $this->allowedGroups = $allowedGroups;
    }

    /**
     * Creates a local User from Active Directory.
     * @param User $user
     * @param string $password
     * @return \Illuminate\Database\Eloquent\Model
     */
    protected function getModelFromAdldap(User $user, $password)
    {

        // Check if the user is in the right group before building the model
        if (!$this->checkLDAPGroups($user)) {

            return;
        }

        return parent::getModelFromAdldap($user, $password);
    }

    /**
     * Check if the specified user is in the specified LDAP group
     * @param $authenticatedUser
     * @return bool
     * @throws LdapGroupException
     */
    private function checkLDAPGroups(User $authenticatedUser)
    {

        //If no groups have been set, skip the check
        if (empty($this->allowedGroups)) {
            return true;
        }

        foreach ($this->allowedGroups as $group) {
            // Check with LDAP if this user has group needed on LDAP
            if ($authenticatedUser->inGroup($group)) {
                return true;
            }
        }

        // If the user is not in the correct group, then we need to check the
        // users table to see if they have been given explicit access. At
        // this point the user has already been authorized by ldap
        $this->checkIfLdapUserHasBeenGivenAccess($authenticatedUser);

        return true;

    }

    /**
     * Retrieve a user by the given credentials.
     * @param $authenticatedUser
     * @return Authenticatable|null
     * @throws LdapGroupException
     */
    public function checkIfLdapUserHasBeenGivenAccess(User $authenticatedUser)
    {
        $credentials = $this->getDatabaseCredentials($authenticatedUser);

        $query = $this->createModel()->newQuery();

        foreach ($credentials as $key => $value) {
            if (!Str::contains($key, 'password')) {
                $query->where($key, $value);
            }
        }

        $user = $query->first();

        if (is_null($user)) {
            throw new LdapGroupException('You do not have permission to access this CMS.');
        }
    }

    /**
     * @param mixed $allowedGroups
     */
    public function setAllowedGroups($allowedGroups)
    {
        $this->allowedGroups = $allowedGroups;
    }

    /**
     * Get the Ldap credentials for a database lookup
     * @param User $authenticatedUser
     * @return array
     */
    protected function getDatabaseCredentials(User $authenticatedUser)
    {
        // Get the username input attributes
        $attributes = $this->getUsernameAttribute();
        // Get the input key
        $key = key($attributes);
        // Get the username from the AD model.
        $username = $authenticatedUser->{$attributes[$key]};

        // Make sure we retrieve the first username
        // result if it's an array.
        if (is_array($username)) {
            $username = Arr::get($username, 0);
        }
        $credentials = [$key => $username];

        return $credentials;
    }

}